﻿namespace tSNE

open FsAlg.Generic

open MatrixFun

/// computation of low-dimensional propability matrix
module LowDimProb =

    /// computation of low-dimensional propability matrix
    let Calculate n (ys : Matrix<double>) =         
        
        let ysSum =  
            ys .* ys
            |> MatrixFun.rowSum 
            |> MatrixFun.ofArray n
            |> Matrix.transpose
           
        // formula of student t-distribution
        let tSq = -2.0 * (ys * (ys |> Matrix.transpose))
        let num = 1.0 + (ysSum + ((ysSum |> Matrix.transpose) + tSq)) 
        let tDis = 1.0 / num // ^ -1
        
        // add diagonal zeros
        let probs = 
           Matrix.init n n (fun i j ->
                match i = j with 
                | true -> 0.0
                | false -> tDis.[i,j]
            )
        
        // normalization of t-Distribution
        let tDis = probs / (probs |> MatrixFun.sum)

        (tDis,probs)