﻿namespace tSNE

open Aardvark.Base
open FsAlg.Generic
open FSharp.Data
open System
open System.IO
open System.Globalization

open MatrixFun

/// Visualizing Data using t-SNE (paper implementation)
/// (paper: http://www.jmlr.org/papers/volume9/vandermaaten08a/vandermaaten08a.pdf) 
module tSNE =
    
    /// reads csv-file and returns ground truth and data matrices
    let readCsv (p : string) =
        let p = CsvFile.Load(p).Cache()
        let t = p.Rows |> Seq.map ( fun r -> r.GetColumn "class" ) |> Seq.toArray
        let rows = 
            p.Rows
            |> Seq.map ( fun r -> 
                r.Columns 
                |> Array.skip 1
                |> Array.map ( fun x -> Double.Parse(x, CultureInfo.InvariantCulture) )
            ) |> Seq.toArray
        let n = p.Rows |> Seq.length
        let X = Array2D.init n (p.NumberOfColumns - 1) ( fun i j -> rows.[i].[j] ) |> Matrix
        (t, X, n)

    /// parses input data and runs tSNE algorithm
    let Run (csv : string) (title : string) (o : string) =

        printfn "reading data from csv ..."
        let (t,X,n) = csv |> readCsv
        
        // we're working with distances from normalized input points
        let disSquared = X |> MatrixFun.normalize |> MatrixFun.squaredDistances n

        // default params
        let perp = 30.0 // perplexity
        let T = 1000 // number of iterations
        let η = 500.0 // learning rate
        let α = 0.5 // momentum tern 

        // run the tSNE algorithm 
        let tSneSimple = Algorithm.tSNEsimpleVersion perp T η n disSquared

        // create html file with visualization
        Visualization.createHtml title o t tSneSimple
